@@ -8,7 +8,7 @@
  *
  * This implements functions that manipulate messages
  *
- * $Id: imap_messages.php,v 1.75 2002/01/12 11:15:12 brong Exp $
+ * $Id: imap_messages.php,v 1.76 2002/01/29 00:07:50 kink Exp $
  */
 
 /****************************************************************************
@@ -245,7 +245,7 @@ function sqimap_get_small_header_list ($imap_stream, $msg_list, $issent) {
 function sqimap_get_flags ($imap_stream, $i) {
     $read = sqimap_run_command ($imap_stream, "FETCH $i:$i FLAGS", true, $response, $message);
     if (ereg("FLAGS(.*)", $read[0], $regs)) {
-        return explode(" ", trim(ereg_replace('[\\(\\)\\\\]', '', $regs[1])));
+        return explode(' ', trim(ereg_replace('[\\(\\)\\\\]', '', $regs[1])));
     }
     return Array('None');
 }
