@@ -9,30 +9,32 @@
  * This contains functions that display mailbox information, such as the
  * table row that has sender, date, subject, etc...
  *
- * $Id: mailbox_display.php,v 1.208 2002/01/19 13:49:35 teepe Exp $
+ * $Id: mailbox_display.php,v 1.218 2002/02/19 01:43:19 brong Exp $
  */
 
-include_once('../functions/strings.php');
+require_once('../functions/strings.php');
 
 define('PG_SEL_MAX', 10);  /* Default value for page_selector_max. */
 
 function printMessageInfo($imapConnection, $t, $i, $key, $mailbox, $sort, $start_msg, $where, $what) {
-    global $checkall;
-    global $color, $msgs, $msort;
-    global $sent_folder, $draft_folder;
-    global $default_use_priority;
-    global $message_highlight_list;
-    global $index_order;
-    global $pos;            /* Search postion (if any)  */
+    global $checkall,
+           $color, $msgs, $msort,
+           $sent_folder, $draft_folder,
+           $default_use_priority,
+           $message_highlight_list,
+           $index_order,
+           $pos;            /* Search postion (if any)  */
 
     $color_string = $color[4];
     if ($GLOBALS['alt_index_colors']) {
-        if (!isset($GLOBALS["row_count"])) {
-            $GLOBALS["row_count"] = 0;
+        if (!isset($GLOBALS['row_count'])) {
+            $GLOBALS['row_count'] = 0;
         }
-        $GLOBALS["row_count"]++;
-        if ($GLOBALS["row_count"] % 2) {
-        if (!isset($color[12])) $color[12] = '#EAEAEA';
+        $GLOBALS['row_count']++;
+        if ($GLOBALS['row_count'] % 2) {
+            if (!isset($color[12])) {
+                $color[12] = '#EAEAEA';
+            }
             $color_string = $color[12];
         }
     }
@@ -115,7 +117,7 @@ function printMessageInfo($imapConnection, $t, $i, $key, $mailbox, $sort, $start
         if( !isset( $pos ) || $pos == '' ) {
             $pos = '0';
         }
-        $search_stuff = "&pos=" . urlencode( $pos ) . "&where=".urlencode($where).'&what='.urlencode($what);
+        $search_stuff = "&amp;pos=" . urlencode( $pos ) . "&amp;where=".urlencode($where).'&amp;what='.urlencode($what);
     }
 
     $checked = ($checkall == 1 ?' checked' : '');
@@ -134,7 +136,7 @@ function printMessageInfo($imapConnection, $t, $i, $key, $mailbox, $sort, $start
         case 4: /* subject */
             echo "   <td bgcolor=\"$hlt_color\">$bold";
                 if (! isset($search_stuff)) { $search_stuff = ''; }
-            echo "<a href=\"read_body.php?mailbox=$urlMailbox&passed_id=".$msg["ID"]."&startMessage=$start_msg&show_more=0$search_stuff\"";
+            echo "<a href=\"read_body.php?mailbox=$urlMailbox&amp;passed_id=".$msg["ID"]."&amp;startMessage=$start_msg&amp;show_more=0$search_stuff\"";
             do_hook("subject_link");
 
             if ($subject != $msg['SUBJECT']) {
@@ -151,7 +153,7 @@ function printMessageInfo($imapConnection, $t, $i, $key, $mailbox, $sort, $start
             echo "   <td bgcolor=\"$hlt_color\" align=center nowrap><b><small>\n";
             if (isset($msg['FLAG_ANSWERED']) &&
                 $msg['FLAG_ANSWERED'] == true) {
-                echo "A\n";
+                echo _("A") . "\n";
                 $stuff = true;
             }
             if ($msg['TYPE0'] == 'multipart') {
@@ -266,7 +268,7 @@ function showMessagesForMailbox
 
         while ($j < $end_loop) {
             if (isset($date[$j])) {
-                $date[$j] = ereg_replace('  ', ' ', $date[$j]);
+                $date[$j] = str_replace('  ', ' ', $date[$j]);
                 $tmpdate = explode(' ', trim($date[$j]));
             } else {
                 $tmpdate = $date = array('', '', '', '', '', '');
@@ -404,7 +406,7 @@ function displayMessageArray($imapConnection, $num_msgs, $start_msg, &$msgs, $ms
     }
 
     mail_message_listing_beginning( $imapConnection,
-        "move_messages.php?msg=$msg&mailbox=$urlMailbox&startMessage=$start_msg",
+        "move_messages.php?msg=$msg&amp;mailbox=$urlMailbox&amp;startMessage=$start_msg",
         $mailbox, $sort, $msg_cnt_str, $paginator_str, $start_msg);
 
     $groupNum = $start_msg % ($show_num - 1);
@@ -520,9 +522,12 @@ function mail_message_listing_beginning
 
     $boxes = sqimap_mailbox_list($imapConnection);
     for ($i = 0; $i < count($boxes); $i++) {
-        if (!in_array("noselect", $boxes[$i]['flags'])) {
+        if (!in_array('noselect', $boxes[$i]['flags'])) {
             $box = $boxes[$i]['unformatted'];
             $box2 = str_replace(' ', '&nbsp;', $boxes[$i]['unformatted-disp']);
+            if( $box2 == 'INBOX' ) {
+                $box2 = _("INBOX");
+            }
             echo "         <OPTION VALUE=\"$box\">$box2</option>\n";
         }
     }
@@ -543,11 +548,11 @@ function mail_message_listing_beginning
     echo '</TD></TR>'.
 
         "<TR><TD BGCOLOR=\"$color[0]\">".
-        "<TABLE WIDTH=\"100%\" BORDER=0 CELLPADDING=2 CELLSPACING=";
+        '<TABLE WIDTH="100%" BORDER=0 CELLPADDING=2 CELLSPACING=';
     if ($GLOBALS['alt_index_colors']) {
-        echo "0";
+        echo '0';
     } else {
-        echo "1";
+        echo '1';
     }
     echo " BGCOLOR=\"$color[0]\">".
         "<TR BGCOLOR=\"$color[5]\" ALIGN=\"center\">";
@@ -609,7 +614,7 @@ function ShowSortButton($sort, $mailbox, $Up, $Down) {
 
     /* Now that we have everything figured out, show the actual button. */
     echo ' <a href="right_main.php?newsort=' . $which .
-        '&startMessage=1&mailbox=' . urlencode($mailbox) .
+        '&amp;startMessage=1&amp;mailbox=' . urlencode($mailbox) .
         '"><IMG SRC="../images/' . $img .
         '" BORDER=0 WIDTH=12 HEIGHT=10></a>';
 }
@@ -618,9 +623,10 @@ function get_selectall_link($start_msg, $sort) {
     global $checkall, $what, $where, $mailbox, $javascript_on;
     global $PHP_SELF, $PG_SHOWNUM;
 
+    $result = '';
     if ($javascript_on) {
         $result =
-            '<script language="JavaScript">' .
+            '<script language="JavaScript" type="text/javascript">' .
             "\n<!-- \n" .
             "function CheckAll() {\n" .
             "   for (var i = 0; i < document.messageList.elements.length; i++) {\n" .
@@ -632,8 +638,14 @@ function get_selectall_link($start_msg, $sort) {
             "//-->\n" .
             '</script><a href="#" onClick="CheckAll();">' . _("Toggle All") . "</a>\n";
     } else {
-        $result .= "<a href=\"$PHP_SELF?mailbox=" . urlencode($mailbox)
-                    . "&startMessage=$start_msg&sort=$sort&checkall=";
+        if (strpos($PHP_SELF, "?")) {
+            $result .= "<a href=\"$PHP_SELF&amp;mailbox=" . urlencode($mailbox)
+                    . "&amp;startMessage=$start_msg&amp;sort=$sort&amp;checkall=";
+        }
+		else {
+            $result .= "<a href=\"$PHP_SELF?mailbox=" . urlencode($mailbox)
+                    . "&amp;startMessage=$start_msg&amp;sort=$sort&amp;checkall=";
+        }
         if (isset($checkall) && $checkall == '1') {
             $result .= '0';
         } else {
@@ -641,12 +653,12 @@ function get_selectall_link($start_msg, $sort) {
         }
 
         if (isset($where) && isset($what)) {
-            $result .= '&where=' . urlencode($where)
-                    . '&what=' . urlencode($what);
+            $result .= '&amp;where=' . urlencode($where)
+                    . '&amp;what=' . urlencode($what);
         }
 
         if ($PG_SHOWNUM == 999999) {
-            $result .= '&PG_SHOWNUM=999999';
+            $result .= '&amp;PG_SHOWNUM=999999';
         }
 
         $result .= "\">";
@@ -688,7 +700,7 @@ function get_msgcnt_str($start_msg, $end_msg, $num_msgs) {
 function get_paginator_link
 ($box, $start_msg, $use, $text) {
     $result = "<A HREF=\"right_main.php?use_mailbox_cache=$use"
-            . "&startMessage=$start_msg&mailbox=$box\" "
+            . "&amp;startMessage=$start_msg&amp;mailbox=$box\" "
             . "TARGET=\"right\">$text</A>";
     return ($result);
 }
@@ -857,14 +869,14 @@ function get_paginator_str
         }
     } else if ($PG_SHOWNUM == 999999) {
         $pg_str = "<A HREF=\"right_main.php?use_mailbox_cache=$use"
-                . "&startMessage=1&mailbox=$box\" TARGET=\"right\">"
+                . "&amp;startMessage=1&amp;mailbox=$box\" TARGET=\"right\">"
                 . _("Paginate") . '</A>' . $spc;
     }
 
     /* If necessary, compute the 'show all' string. */
     if (($prv_str != '') || ($nxt_str != '')) {
         $all_str = "<A HREF=\"right_main.php?PG_SHOWNUM=999999"
-                 . "&use_mailbox_cache=$use&startMessage=1&mailbox=$box\" "
+                 . "&amp;use_mailbox_cache=$use&amp;startMessage=1&amp;mailbox=$box\" "
                  . "TARGET=\"right\">" . _("Show All") . '</A>';
     }
 
