@@ -6,7 +6,7 @@
  * Copyright (c) 1999-2002 The SquirrelMail Project Team
  * Licensed under the GNU GPL. For full terms see the file COPYING.
  *
- * $Id: printer_friendly_main.php,v 1.6 2002/01/02 09:06:58 philippe_mingo Exp $
+ * $Id: printer_friendly_main.php,v 1.7 2002/02/19 12:31:12 brong Exp $
  */
 
 require_once('../src/validate.php');
@@ -17,10 +17,10 @@ displayHtmlHeader( _("Printer Friendly"), '', FALSE );
 echo "<frameset rows=\"60, *\" noresize border=\"0\">\n".
      "<frame src=\"printer_friendly_top.php\" name=\"top_frame\" scrolling=\"off\">".
      '<frame src="printer_friendly_bottom.php?passed_ent_id='.
-     $passed_ent_id . '&mailbox=' . urlencode($mailbox) .
-     '&passed_id=' . $passed_id .
+     $passed_ent_id . '&amp;mailbox=' . urlencode($mailbox) .
+     '&amp;passed_id=' . $passed_id .
      "\" name=\"bottom_frame\">".
      "</frameset>\n".
      "</html>\n";
 
-?>
\ No newline at end of file
+?>
