@@ -9,7 +9,7 @@
  * top frame of printer_friendly_main.php
  * displays some javascript buttons for printing & closing
  *
- * $Id: printer_friendly_top.php,v 1.6 2002/01/02 09:06:58 philippe_mingo Exp $
+ * $Id: printer_friendly_top.php,v 1.7 2002/02/19 01:49:37 brong Exp $
  */
 
 require_once('../src/validate.php');
@@ -19,7 +19,7 @@ require_once('../src/load_prefs.php');
 require_once('../functions/page_header.php');
 
 displayHtmlHeader( _("Printer Friendly"),
-             "<script language=\"javascript\">\n".
+             "<script language=\"javascript\" type=\"text/javascript\">\n".
              "<!--\n".
              "function printPopup() {\n".
                 "parent.frames[1].focus();\n".
@@ -41,4 +41,4 @@ echo "<body text=\"$color[8]\" bgcolor=\"$color[3]\" link=\"$color[7]\" vlink=\"
      '</body>'.
      "</html>\n";
 
-?>
\ No newline at end of file
+?>
