@@ -9,7 +9,7 @@
  * This is where the mailboxes are listed. This controls most of what
  * goes on in SquirrelMail.
  *
- * $Id: right_main.php,v 1.67 2002/01/09 10:12:43 philippe_mingo Exp $
+ * $Id: right_main.php,v 1.71 2002/02/17 18:32:54 teepe Exp $
  */
 
 require_once('../src/validate.php');
@@ -104,9 +104,11 @@ if (! isset($use_mailbox_cache)) {
 }
 
 /* There is a problem with registered vars in 4.1 */
+/*
 if( substr( phpversion(), 0, 3 ) == '4.1'  ) {
     $use_mailbox_cache = FALSE;
 }
+*/
 
 if ($use_mailbox_cache && session_is_registered('msgs')) {
     showMessagesForMailbox($imapConnection, $mailbox, $numMessages, $startMessage, $sort, $color, $show_num, $use_mailbox_cache);
@@ -131,13 +133,16 @@ if ($use_mailbox_cache && session_is_registered('msgs')) {
 
     if (session_is_registered('msgs') && isset($msgs)) {
         session_register('msgs');
+        $_SESSION['msgs'] = $msgs;
     }
 
     if (session_is_registered('msort') && isset($msort)) {
         session_register('msort');
+        $_SESSION['msort'] = $msort;
     }
 
     session_register('numMessages');
+    $_SESSION['numMessages'] = $numMessages;
 }
 
 do_hook('right_main_bottom');
